(* $Id: draw.ml,v 1.5 2006/05/28 18:34:46 deraugla Exp $
 *
 * Rogloglo Toolkit: drawing routines
 *)

open Xlib;
open Std;
open Rtdecl;
open Util;

value rt_draw_point draw (x, y) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawPoint (xd.dpy, draw, xd.gc, x, y)
and rt_draw_line draw (x1, y1) (x2, y2) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawLine (xd.dpy, draw, xd.gc, x1, y1, x2, y2)
;

value pts =
  let plen = ref 20 in
  let ptsr = ref (alloc_XPoint plen.val) in
  fun len -> (
    if len > plen.val then (
      free ptsr.val;
      ptsr.val := alloc_XPoint len;
      plen.val := len
    )
    else ();
    ptsr.val
  )
;

value rt_draw_lines draw points = (
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  let pts = pts (List.length points) in
  do_list_i
    (fun i (x, y) -> (
       let ii = i in
       set_XPoint_x (x, pts, ii);
       set_XPoint_y (y, pts, ii)
     ))
    0 points;
  xDrawLines (xd.dpy, draw, xd.gc, pts, List.length points, coordModeOrigin)
)
and rt_fill_polygon draw points = (
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  let pts = pts (List.length points) in
  do_list_i
    (fun i (x, y) -> (
       let ii = i in
       set_XPoint_x (x, pts, ii);
       set_XPoint_y (y, pts, ii)
     ))
    0 points;
  xFillPolygon
    (xd.dpy, draw, xd.gc, pts, List.length points, convex, coordModeOrigin)
);

value rt_fill_rectangle draw (x, y, width, height) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xFillRectangle (xd.dpy, draw, xd.gc, x, y, width, height)
and rt_draw_rectangle draw (x, y, width, height) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawRectangle (xd.dpy, draw, xd.gc, x, y, width, height)
and rt_fill_arc draw (x, y, width, height, a1, a2) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xFillArc (xd.dpy, draw, xd.gc, x, y, width, height, a1, a2)
and rt_draw_arc draw (x, y, width, height, a1, a2) =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawArc (xd.dpy, draw, xd.gc, x, y, width, height, a1, a2)
and rt_clear_area wid (x, y, width, height) =
  let xd = wid.wid_xd in
  xClearArea (xd.dpy, wid.win, x, y, width, height, 0)
and rt_clear_widget wid =
  let xd = wid.wid_xd in
  xClearWindow (xd.dpy, wid.win)
and rt_erase_draw_string draw (x, y) str =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawImageString (xd.dpy, draw, xd.gc, x, y, str, String.length str)
and rt_draw_string draw (x, y) str =
  let (xd, draw) =
    match draw with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  xDrawString (xd.dpy, draw, xd.gc, x, y, str, String.length str)
;

value rt_copy_area draw1 draw2 (src_x, src_y, width, height) (dst_x, dst_y) =
(
  let (xd1, draw1) =
    match draw1 with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  and (xd2, draw2) =
    match draw2 with
    [ WidgetDr wid -> (wid.wid_xd, wid.win)
    | PixmapDr pixm -> (pixm.pixm_xd, pixm.pixmap) ]
  in
  if xd1 != xd2 then
    failwith "rt_copy_area: can't copy between different displays"
  else ();
  let xd = xd1 in
  xCopyArea
    (xd.dpy, draw1, draw2, xd.gc, src_x, src_y, width, height, dst_x, dst_y)
);

value rt_set_line_width xd lw =
  xSetLineAttributes (xd.dpy, xd.gc, lw, lineSolid, capRound, joinMiter)
;

value rt_set_backing_store =
  let xswa = mallocated_var alloc_XSetWindowAttributes (ref None) in
  fun wid -> (
    let xswa = xswa () in
    set_XSetWindowAttributes_backing_store (always, xswa);
    xChangeWindowAttributes (wid.wid_xd.dpy, wid.win, cWBackingStore, xswa)
  )
;
